@@ -1,15 +1,18 @@
+import json
 from dataclasses import dataclass
-from typing import Dict, List, Optional, TypedDict, Union
+from datetime import datetime
+from typing import cast, Dict, List, Optional, TypedDict, Union
 
 import dateutil.parser
 import termcolor
+
 from zuper_commons.timing import now_utc
 from zuper_commons.types import ZValueError
-
 from .challenge import ChallengeDescription, EvaluationParametersDict
 from .challenges_constants import ChallengesConstants
 from .rest import make_server_request
 from .types import (
+    ArtefactID,
     ChallengeID,
     ChallengeName,
     ChallengeStepID,
@@ -120,7 +123,7 @@ class UserInfoResponseDict(TypedDict):
 
 
 def get_dtserver_user_info(token: str, impersonate: Optional[UserID] = None) -> UserInfoResponseDict:
-    """ Returns a dictionary with information about the user """
+    """Returns a dictionary with information about the user"""
     endpoint = Endpoints.user_info
     method = "GET"
     data: UserInfoRequestDict
@@ -174,40 +177,20 @@ def dtserver_retire_same_label(
 
 
 class GetUserSubmissionRequestDict(TypedDict):
-    pass
-
-
-#
-# class  GetUserSubmissionResponseDict(TypedDict):
-#     pass
-
-
-def dtserver_get_user_submissions(token: str, impersonate: Optional[UserID] = None):
-    """ Returns a dictionary with information about the user submissions """
-    endpoint = Endpoints.submissions
-    method = "GET"
-    data: GetUserSubmissionRequestDict
-    data = {}
-    add_version_info(data)
-    add_impersonate_info(data, impersonate)
-
-    submissions = make_server_request(token, endpoint, data=data, method=method)
-    # submissions = cast(GetUserSubmissionResponseDict, submissions)
-    for v in submissions.values():
-        for k in ["date_submitted", "last_status_change"]:
-            v[k] = dateutil.parser.parse(v[k])
-    return submissions
+    challenge_name: ChallengeName
 
 
 class SubmissionDict(TypedDict):
     submission_id: SubmissionID
     complete: bool
     status: str
-    challenge_id: int
-    challenge_name: str
+    challenge_id: ChallengeID
+    challenge_name: ChallengeName
     challenge_is_open: int
     user_label: str
     user_metadata: str
+    last_status_change: datetime
+    date_submitted: datetime
 
 
 # dict[11]
@@ -224,13 +207,36 @@ class SubmissionDict(TypedDict):
 #       │ │ user_metadata: {}
 
 
+def dtserver_get_user_submissions(
+    token: str, impersonate: Optional[UserID] = None, challenge_name: Optional[ChallengeName] = None
+) -> Dict[SubmissionID, SubmissionDict]:
+    """Returns a dictionary with information about the user submissions"""
+    endpoint = Endpoints.submissions
+    method = "GET"
+    data: GetUserSubmissionRequestDict
+    data = {}
+    if challenge_name:
+        data["challenge_name"] = challenge_name
+    print(json.dumps(data, indent=3))
+    add_version_info(data)
+    add_impersonate_info(data, impersonate)
+
+    submissions = make_server_request(token, endpoint, data=data, method=method)
+    submissions = {int(k): v for k, v in submissions.items()}
+    # submissions = cast(Dict[SubmissionID, GetUserSubmissionResponseDict], submissions)
+    for v in submissions.values():
+        for k in ["date_submitted", "last_status_change"]:
+            v[k] = dateutil.parser.parse(v[k])
+    return submissions
+
+
 def dtserver_get_submissions(
     token: str,
     challenge_name: Optional[ChallengeName],
     user_id: Optional[UserID],
     impersonate: Optional[UserID] = None,
-):
-    """ Returns a dictionary with information about the submissions """
+) -> Dict[SubmissionID, SubmissionDict]:
+    """Returns a dictionary with information about the submissions"""
     endpoint = Endpoints.submissions
     method = "GET"
     data: GetUserSubmissionRequestDict
@@ -244,6 +250,7 @@ def dtserver_get_submissions(
         query_string["user_id"] = user_id
 
     submissions = make_server_request(token, endpoint, data=data, method=method, query_string=query_string)
+    submissions = {int(k): v for k, v in submissions.items()}
     # submissions = cast(GetUserSubmissionResponseDict, submissions)
     for v in submissions.values():
         for k in ["date_submitted", "last_status_change"]:
@@ -279,7 +286,7 @@ class ChallengeInfoDict(TypedDict):
     queue_name: ChallengeName  ## XXX: redundan
 
 
-class SubmissionDict(TypedDict):
+class SubmissionResponseDict(TypedDict):
     submission_id: SubmissionID
     existing: bool
     challenge: ChallengeInfoDict
@@ -287,7 +294,7 @@ class SubmissionDict(TypedDict):
 
 class Submit2ResponseDict(TypedDict):
     component_id: int
-    submissions: Dict[ChallengeName, SubmissionDict]
+    submissions: Dict[ChallengeName, SubmissionResponseDict]
 
 
 class AddSubmissionRequest(TypedDict):
@@ -314,7 +321,13 @@ def dtserver_submit2(
     return make_server_request(token, endpoint, data=data, method=method)
 
 
+class GetInfoResponse(TypedDict):
+    pass
+
+
 def dtserver_get_info(token, submission_id: SubmissionID, impersonate: Optional[UserID] = None):
+    if submission_id is None:
+        raise TypeError(type(submission_id))
     endpoint = Endpoints.submission_single + f"/{submission_id}"
     method = "GET"
     data = {}
@@ -362,25 +375,25 @@ def dtserver_report_job(
     impersonate: Optional[UserID] = None,
 ):
     """
-        result: JobStatusString, one of ChallengesConstants.ALLOWED_JOB_STATUS
-            success, failed, error, aborted, host-error
+    result: JobStatusString, one of ChallengesConstants.ALLOWED_JOB_STATUS
+        success, failed, error, aborted, host-error
 
-        uploaded: structure returned by upload_files(directory, aws_config)
-         which uses S3
+    uploaded: structure returned by upload_files(directory, aws_config)
+     which uses S3
 
-        ipfs_hashes: the files represented by IPFS
-            filename -> IPFS hash
+    ipfs_hashes: the files represented by IPFS
+        filename -> IPFS hash
 
-            keep empty
+        keep empty
 
 
-        for example :
-            status = 'success'
-            stats = {'msg': 'ok', 'scores': {}}
+    for example :
+        status = 'success'
+        stats = {'msg': 'ok', 'scores': {}}
 
 
-            status = 'failed'
-            stats = {'msg': 'why it failed', 'scores': {}}
+        status = 'failed'
+        stats = {'msg': 'why it failed', 'scores': {}}
     """
     endpoint = Endpoints.take_submission
     method = "POST"
@@ -397,7 +410,12 @@ def dtserver_report_job(
     add_version_info(data)
     add_impersonate_info(data, impersonate)
     return make_server_request(
-        token, endpoint, data=data, method=method, timeout=timeout, suppress_user_msg=True,
+        token,
+        endpoint,
+        data=data,
+        method=method,
+        timeout=timeout,
+        suppress_user_msg=True,
     )
 
 
@@ -419,7 +437,7 @@ class UploadStorage(TypedDict):
     s3: S3ObjectDict
 
 
-@dataclass
+# @dataclass
 class ArtefactDict(TypedDict):
     size: int
     mime_type: str
@@ -428,6 +446,10 @@ class ArtefactDict(TypedDict):
     storage: UploadStorage
 
 
+class ArtefactDictWithID(ArtefactDict):
+    artefact_id: Optional[ArtefactID]
+
+
 EvaluatorFeaturesDict = Dict[str, Union[str, int, float, bool]]
 
 
@@ -517,67 +539,67 @@ def dtserver_work_submission(
     impersonate: Optional[UserID] = None,
 ) -> WorkSubmissionResultDict:
     """
-        token: Duckietown token
+    token: Duckietown token
 
-        submission_id: if None, get any submission. If set, try to get that one.
-        reset: if submission already evaluated, reset the previous jobs.
+    submission_id: if None, get any submission. If set, try to get that one.
+    reset: if submission already evaluated, reset the previous jobs.
 
-        features: dictionary of "features - these are well known
+    features: dictionary of "features - these are well known
 
-        example:
+    example:
 
-            features:
-                map_aido5_large_loop: 1
-                nduckiebots: 3
-                nduckies: 20
+        features:
+            map_aido5_large_loop: 1
+            nduckiebots: 3
+            nduckies: 20
 
-        machine_id, process_id, evaluator_version: identify the identity of the evaluator.
-        machine_id, process_id: free form - valid identifier
-        evaluator_version = duckietown_challenges_runner.__version__
+    machine_id, process_id, evaluator_version: identify the identity of the evaluator.
+    machine_id, process_id: free form - valid identifier
+    evaluator_version = duckietown_challenges_runner.__version__
 
 
-        timeout: connection timeout
-        impersonate: ID to impersonate if any
+    timeout: connection timeout
+    impersonate: ID to impersonate if any
 
 
-        Pipeline:
+    Pipeline:
 
-        1) get a job using
+    1) get a job using
 
-            res = dtserver_work_submission(...)
+        res = dtserver_work_submission(...)
 
-        2) Take
+    2) Take
 
-            aws_config = res['aws_config']
+        aws_config = res['aws_config']
 
-        3) Use upload_files:
+    3) Use upload_files:
 
-            uploaded = upload_files(directory, aws_config)
+        uploaded = upload_files(directory, aws_config)
 
-        4) Use your code to upload IPFS, get
+    4) Use your code to upload IPFS, get
 
-            ipfs_hashes = {
-                'rel/filename' : '/ipfs/<hash>'
-            }
+        ipfs_hashes = {
+            'rel/filename' : '/ipfs/<hash>'
+        }
 
-        5) Put the statistics in a "scores" dictionary.
+    5) Put the statistics in a "scores" dictionary.
 
-            stats = {'metric1': 1.0, 'metric2': 2.0}
+        stats = {'metric1': 1.0, 'metric2': 2.0}
 
-        5) Give this structure to the server using
+    5) Give this structure to the server using
 
-            dtserver_report_job(...,
-                uploaded=uploaded,
-                ipfs_hashes_hashes,
-                stats=stats)
+        dtserver_report_job(...,
+            uploaded=uploaded,
+            ipfs_hashes_hashes,
+            stats=stats)
 
 
 
-        Used to get a job from the server.
+    Used to get a job from the server.
 
-        Returns a dict containing among others.
+    Returns a dict containing among others.
 
-            aws_config: credentials to pass to upload_files
+        aws_config: credentials to pass to upload_files
 
     """
     endpoint = Endpoints.take_submission
@@ -594,7 +616,12 @@ def dtserver_work_submission(
     add_version_info(data)
     add_impersonate_info(data, impersonate)
     return make_server_request(
-        token, endpoint, data=data, method=method, timeout=timeout, suppress_user_msg=True,
+        token,
+        endpoint,
+        data=data,
+        method=method,
+        timeout=timeout,
+        suppress_user_msg=True,
     )
 
 
@@ -626,9 +653,9 @@ def dtserver_job_heartbeat(
     query_string: str = None,
 ) -> HeartbeatResponseDict:
     """
-        You just call every 30 seconds.
+    You just call every 30 seconds.
 
-        Optionally you can already send some uploaded files.
+    Optionally you can already send some uploaded files.
 
 
     """
@@ -700,7 +727,11 @@ class JobInfoDict(TypedDict):
     why: Optional[str]
 
 
-def dtserver_get_job(token: str, job_id: JobID, impersonate: Optional[UserID] = None,) -> JobInfoDict:
+def dtserver_get_job(
+    token: str,
+    job_id: JobID,
+    impersonate: Optional[UserID] = None,
+) -> JobInfoDict:
     endpoint = Endpoints.jobs + f"/{job_id}"
     method = "GET"
     data = {}
@@ -718,15 +749,27 @@ def add_version_info(data):
         pass
 
 
-# noinspection PyUnresolvedReferences,PyBroadException,PyCompatibility,PyProtectedMember
-def get_packages_version() -> Dict[str, VersionInfo]:
-    try:
-        from pip import get_installed_distributions
-    except:
-        from pip._internal.utils.misc import get_installed_distributions
+def _get_installed_distributions(
+    local_only: bool = True,
+    user_only: bool = False,
+    paths: Optional[List[str]] = None,
+):
+    """Return a list of installed Distribution objects."""
+    from pip._internal.metadata import get_environment
+    from pip._internal.metadata.pkg_resources import Distribution as _Dist
 
+    env = get_environment(paths)
+    dists = env.iter_installed_distributions(
+        local_only=local_only,
+        user_only=user_only,
+        skip=[],
+    )
+    return [cast(_Dist, dist)._dist for dist in dists]
+
+
+def get_packages_version() -> Dict[str, VersionInfo]:
     packages = {}
-    for i in get_installed_distributions(local_only=False):
+    for i in _get_installed_distributions(local_only=False):
         pkg = {"version": i._version, "location": i.location}
         packages[i.project_name] = pkg
 
